#!/usr/bin/env python
"""
    clbd.tap - this is part of clb (cloud load balancer)

    Copyright 2010, Canonical Ltd.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
from zope.interface import implements

from twisted.python import usage
from twisted.web import server, resource
from twisted.application import internet
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker
import sqlite3

from clb import register

class Options(usage.Options):
    optParameters = [['database', 'd', register.serverdb,"Database to connect to"],
                     ['port', 'p', 8090, "Port to listen for requests on"],
                     ['address', 'a', '0.0.0.0', "Address to bind to"]]

class ClbdServiceMaker(object):
    implements(IServiceMaker, IPlugin)
    tapname = "clbd"
    description = "Cloud Load Balancer"
    options = Options

    def makeService(self, options):
        db = sqlite3.connect(options['database'])
        cur = db.cursor()
        cur.execute('CREATE TABLE IF NOT EXISTS servers (name varchar(255), service varchar(255), ip varchar(35), port int, primary key (name, service))')
        cur.execute('CREATE TABLE IF NOT EXISTS services (name varchar(255), ip varchar(35), port int, primary key (name))')

        site = server.Site(register.RegisterResource())
        return internet.TCPServer(int(options['port']), site, interface=options['address'])

serviceMaker = ClbdServiceMaker()
